/*
  Tracks data for a single athlete.
*/

import * as sauce from '/shared/sauce/index.mjs';
import wPrimeData from './wPrimeData.mjs';

export class AthleteData {
  sauceData = undefined; // Athlete data from Sauce.
  powerProfile = undefined; // max 90 day efforts from Zwift profile or ZwiftPower.
  messages = []; // Messages to display to user when this athlete is being watched.
  maxPower = {}; // Max recored power for each time.
  category = {
    letter: '',
    zFtpWattsLimit: undefined,
    zFtpWkgLimit: undefined,
    zMapWattsLimit: undefined,
    zMapWkgLimit: undefined,
  };
  _times = [];
  _latestRollTime = 0; // game-time of newest power data added to rolls. Next update should fetch from this time onward.
  _wPrimeData = undefined;

  constructor(times) {
    this._times = times;
  }

  get latestRollTime() {
    return this._latestRollTime;
  }

  get wPrimeBal() {
    return this._wPrimeData?.wPrimeBal;
  }

  get wPrime() {
    return this._wPrimeData?.wPrime;
  }

  get pctWPrime() {
    return this._wPrimeData?.pctWPrime;
  }

  get criticalPower() {
    return this._wPrimeData?.criticalPower;
  }

  assignState(props) {
    this._wPrimeData = new wPrimeData(props.criticalPower, props.wPrime);
    delete props.criticalPower;
    delete props.wPrime;
    Object.assign(this, props);
  }

  importStreams(timeStream, wattsStream) {
    this._times.forEach(t => {
      const peakPower = sauce.power.peakPower(t, timeStream, wattsStream)?.avg();
      this.maxPower[t] = Math.max(this.maxPower[t] || 0, peakPower || 0);
    });
    this._latestRollTime = timeStream[timeStream.length - 1] - this._times[this._times.length - 1];
    this._wPrimeData?.calculateWPrimeBal(timeStream, wattsStream);
  }
}
