import * as echarts from '/pages/deps/src/echarts.mjs';
import * as theme from '/pages/src/echarts-sauce-theme.mjs';

const labelColor = '#bbb';
const lineColor = '#ccc';
const tickColor = '#555';
const tickLineStyle = {
  lineStyle: {
    color: tickColor,
  },
};

echarts.registerTheme('sauce', theme.getTheme('dynamic', { fg: 'intrinsic-inverted', bg: 'intrinsic' }));

export function createPeakPowerChart(el, chartOptions) {
  const chart = echarts.init(el, 'sauce', { renderer: 'svg' });
  const topPad = 10;
  const bottomPad = 10;
  const leftPad = 15;
  const rightPad = 10;

  const series = [{
    id: 'max90day',
    type: 'line',
    symbol: 'none',
    animation: false,
    emphasis: { disabled: true },
    color: '#b30c15',
    lineStyle: { color: '#b30c15' },
    areaStyle: { color: '#b30c15' },
    z: 0,
  }, {
    id: 'max',
    type: 'line',
    symbol: 'none',
    animation: false,
    emphasis: { disabled: true },
    color: '#46f',
    lineStyle: { color: '#46f' },
    areaStyle: { color: '#46f' },
    z: 1,
  }, {
    id: 'current',
    type: 'line',
    smooth: true,
    symbol: 'circle',
    animation: false,
    emphasis: { disabled: true },
    color: '#ddd',
    lineStyle: { color: '#ddd', width: 1, type: 'dotted' },
    z: 4,
  }, {
  //   id: 'zFtpMaxRegression',
  //   type: 'line',
  //   smooth: true,
  //   symbol: 'none',
  //   animation: false,
  //   emphasis: { disabled: true },
  //   color: '#f00',
  //   lineStyle: { color: '#f00' },
  //   z: 2,
  // }, {
  //   id: 'zFtpRegression',
  //   type: 'line',
  //   smooth: true,
  //   symbol: 'none',
  //   animation: false,
  //   emphasis: { disabled: true },
  //   color: '#00f',
  //   lineStyle: { color: '#00f' },
  //   z: 3,
  // }, {
    id: 'zFtpMaxPoint',
    type: 'scatter',
    symbol: 'arrow',
    symbolRotate: 180,
    animation: false,
    emphasis: { disabled: true },
    color: '#f00',
    z: 5,
  }, {
    id: 'zFtpPoint',
    type: 'scatter',
    symbol: 'arrow',
    animation: false,
    emphasis: { disabled: true },
    color: '#00f',
    z: 6,
  }, {
  //   id: 'zMapMaxProjection',
  //   type: 'line',
  //   smooth: true,
  //   symbol: 'none',
  //   animation: false,
  //   emphasis: { disabled: true },
  //   color: '#f00',
  //   lineStyle: { color: '#f00' },
  //   z: 7,
  // }, {
    id: 'zMapMaxPoint',
    type: 'scatter',
    symbol: 'arrow',
    symbolRotate: 180,
    animation: false,
    emphasis: { disabled: true },
    color: '#f00',
    z: 8,
  }, {
  //   id: 'zMapProjection',
  //   type: 'line',
  //   smooth: true,
  //   symbol: 'none',
  //   animation: false,
  //   emphasis: { disabled: true },
  //   color: '#00f',
  //   lineStyle: { color: '#00f' },
  //   z: 9,
  // }, {
    id: 'zMapPoint',
    type: 'scatter',
    symbol: 'arrow',
    animation: false,
    emphasis: { disabled: true },
    color: '#00f',
    z: 10,
  }];

  const options = {
    animation: false,
    color: series.map(f => f.color),
    backgroundColor: '#292929',
    textStyle: {
      color: labelColor,
    },
    legend: { show: false },
    grid: [{
      top: topPad,
      height: `${(100 - topPad - bottomPad)}%`,
      // left: leftPad,
      // right: rightPad,
      backgroundColor: '#292929',
    }],
    minorSplitLine: {
      lineStyle: {
        color: lineColor,
      },
    },
    xAxis: {
      type: 'value',
      min: chartOptions.xMin,
      max: chartOptions.xMax,
      maxInterval: 5 * 60,
      axisLabel: {
        color: labelColor,
        formatter: f => {
          const minutes = Math.floor(f / 60);
          const seconds = f % 60;
          if (seconds === 0) {
            return `${minutes}m`;
          }
          return `${minutes}:${seconds < 10 ? '0' : ''}${seconds}`;
        },
      },
      axisLine: {
        lineStyle: {
          color: lineColor,
        },
      },
      minorTick: tickLineStyle,
      axisTick: tickLineStyle,
      splitLine: tickLineStyle,
    },
    yAxis: [
      {
        type: 'value',
        scale: true,
        axisLabel: {
          color: labelColor,
        },
        axisLine: {
          lineStyle: {
            color: lineColor,
          },
        },
        minorTick: tickLineStyle,
        axisTick: tickLineStyle,
        splitLine: tickLineStyle,
      },
    ],
    series,
    toolbox: { show: false },
  };
  chart.setOption(options);

  chart.updateData = (data, criticalPower) => {
    const mappedSeries = series.map(s => {
      return {
        data: data[s.id] || [],
      };
    });
  
    if (criticalPower) {
      mappedSeries[0].markLine = {
        lineStyle: {
          color: lineColor,
        },
        label: {
          color: '#ffffff',
          textShadowBlur: 0,
          textBorderWidth: 0,
        },
        data: [{
          yAxis: criticalPower,
        }],
      };
    } else {
      mappedSeries[0].markLine = {
        data: []
      };
    }

    chart.setOption({
      series: mappedSeries,
    });
  };

  return chart;
}
