import * as sauce from '/shared/sauce/index.mjs';

export default class wPrimeData {
  _wPrimeBal = 0;
  _cp = 0;
  _wPrime = 0;
  _startTime = 0;
  _accumulator = undefined;

  constructor(criticalPower, wPrime) {
    this._cp = criticalPower;
    this._wPrime = wPrime;
    this._wPrimeBal = wPrime;
    this._accumulator = sauce.power.makeIncWPrimeBalDifferential(criticalPower, wPrime);
  }

  get wPrimeBal() {
    return this._wPrimeBal;
  }

  get wPrime() {
    return this._wPrime;
  }

  get pctWPrime() {
    return Math.round(this._wPrimeBal / this._wPrime * 100);
  }

  get criticalPower() {
    return this._cp;
  }

  calculateWPrimeBal(timeStream, powerStream) {
    if (!this._cp || !this._wPrime) {
      return undefined;
    }

    const cp = this._cp;
    let prevT = 0;

    // find the index in the streams where we left off from last time calculate was called.
    let startIndex = 0;
    for (let ti = timeStream.length - 1; ti >= 0; ti--) {
      if(timeStream[ti] < this._startTime) {
        startIndex = ti + 1;
        prevT = timeStream[ti];
        break;
      }
    }
    this._startTime = timeStream[timeStream.length - 1];

    for (let i = startIndex; i < timeStream.length; i++) {
      const t = timeStream[i];
      const elapsed = (t - prevT) || 0;
      this._wPrimeBal = this._accumulator(powerStream[i], elapsed);
      prevT = t;
    }

    return this._wPrimeBal;
  }

  // // https://www.ncbi.nlm.nih.gov/pmc/articles/PMC6934642/
  // // https://pubmed.ncbi.nlm.nih.gov/24492634/
  // calculateWPrimeBal(timeStream, powerStream) {
  //   if (!this._cp || !this._wPrime) {
  //     return undefined;
  //   }

  //   const cp = this._cp;
  //   let prevT = 0;

  //   let startIndex = 0;
  //   for (let ti = timeStream.length - 1; ti >= 0; ti--) {
  //     if(timeStream[ti] < this._startTime) {
  //       startIndex = ti + 1;
  //       prevT = timeStream[ti];
  //       break;
  //     }
  //   }

  //   for (let i = startIndex; i < timeStream.length; i++) {
  //     const t = timeStream[i];
  //     const p = powerStream[i];
  //     const Dcp = cp - p;
  //     const tau = 546 * Math.E ** (-0.01 * Dcp) + 316;
  //     const tu = t - prevT; // duration of the recovery interval

  //     Math.E ** -(tu / tau) * tu;
  //   }
  // }

  // // https://www.ncbi.nlm.nih.gov/pmc/articles/PMC6934642/
  // calculateWPrimeBal2(timeStream, powerStream) {
  //   if (!this._cp || !this._wPrime) {
  //     return undefined;
  //   }

  //   const cp = this._cp;
  //   let prevT = 0;

  //   let startIndex = 0;
  //   for (let ti = timeStream.length - 1; ti >= 0; ti--) {
  //     if(timeStream[ti] < this._startTime) {
  //       startIndex = ti + 1;
  //       prevT = timeStream[ti];
  //       break;
  //     }
  //   }

  //   for (let i = startIndex; i < timeStream.length; i++) {
  //     const t = timeStream[i];
  //     const p = powerStream[i];
  //     const wp0 = this._wPrimeBal || 1; // avoid div by 0
  //     const Dt = t - prevT; // delta time
  //     let wpDelta = 0; // power exprended in this time period Dt

  //     if (p >= cp) {
  //       wpDelta = ((p - cp) * Dt);
  //     } else {
  //       const Dcp = cp - p;
  //       const wpExp = this._wPrime - wp0;
  //       console.log(`wP: ${this._wPrime}, wp0: ${wp0}, wpExp: ${wpExp}, cp: ${cp}, p: ${p}, Dcp: ${Dcp}, Dt: ${Dt}`);
  //       wpDelta = wpExp * Math.E ** (-Dcp * Dt / wp0);
  //     }

  //     this._wPrimeBal -= wpDelta;

  //     if (this._wPrimeBal > this._wPrime) {
  //       this._wPrimeBal = this._wPrime;
  //     }
  //     console.log(`t: ${t}, p: ${p}, wpDelta: ${wpDelta} wpBal: ${this._wPrimeBal}`);

  //     prevT = t;
  //   }
  //   this._startTime = timeStream[timeStream.length - 1];

  //   return this._wPrimeBal;
  // }
}